
#include <stdint.h>
#include <string.h>
#include "ble_tonepen.h"
#include "ble_srv_common.h"
#include "app_error.h"
#include "SEGGER_RTT.h"

#include "../../../examples/ble_peripheral/ble_app_template/tmd3702.h"

void tonepen_tmd3702_read_callback(ret_code_t result, void * p_user_data) {
    ble_tonepen_t *p_tonepen = (ble_tonepen_t*) p_user_data;
    ble_gatts_value_t p_value;
    p_value.len = 2;
    p_value.offset = 0;
    p_value.p_value = read_data;
    //sd_ble_gatts_value_set(p_tonepen->conn_handle, p_tonepen->tmd3702_read_handles.value_handle, &p_value);
}

static void on_write(ble_tonepen_t * p_tonepen, ble_evt_t const * p_ble_evt)
{
/*
    ble_gatts_evt_write_t const * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if (   (p_evt_write->handle == p_tonepen->tmd3702_write_handles.value_handle)
        && (p_evt_write->len == 2)
        && p_tonepen->twi_mngr != NULL)
    {
        tmd3702_sched_write_reg(p_tonepen->twi_mngr, p_evt_write->data[0], p_evt_write->data[1]);
    } else if (   (p_evt_write->handle == p_tonepen->tmd3702_read_handles.value_handle)
        && (p_evt_write->len == 1)
        && p_tonepen->twi_mngr != NULL)
    {
        tmd3702_sched_read_reg(p_tonepen->twi_mngr, p_evt_write->data[0], p_evt_write->data[1], tonepen_tmd3702_read_callback);
    }
    */
}

void ble_tonepen_on_ble_evt(ble_evt_t const * p_ble_evt, void * p_context)
{
    ble_tonepen_t * p_tonepen = (ble_tonepen_t *)p_context;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GATTS_EVT_WRITE:
            on_write(p_tonepen, p_ble_evt);
            
            break;
        default:
            // No implementation needed.
            break;
    }
}

static void tonepen_add_tmd_characteristics(ble_uuid128_t *base_uuid, ble_tonepen_t * p_tonepen_service) {
    uint32_t err_code;

    ble_uuid_t          char_writ_uuid, char_read_uuid, char_data_uuid;
    char_writ_uuid.uuid      = BLE_UUID_TONEPEN_CHAR_TMD_REG_WRITE_UUID;
    char_read_uuid.uuid      = BLE_UUID_TONEPEN_CHAR_TMD_REG_READ_UUID; 
    char_data_uuid.uuid      = BLE_UUID_TONEPEN_CHAR_TMD_DATA_UUID;

    err_code = sd_ble_uuid_vs_add(&base_uuid, &char_writ_uuid.type);
    APP_ERROR_CHECK(err_code);
    err_code = sd_ble_uuid_vs_add(&base_uuid, &char_read_uuid.type);
    APP_ERROR_CHECK(err_code);
    err_code = sd_ble_uuid_vs_add(&base_uuid, &char_data_uuid.type);
    APP_ERROR_CHECK(err_code);

    /* Config permissions  */
    ble_gatts_char_md_t char_writ_md, char_read_md, char_data_md;
    memset(&char_writ_md, 0, sizeof(char_writ_md));
    memset(&char_read_md, 1, sizeof(char_read_md));
    memset(&char_data_md, 2, sizeof(char_data_md));

    char_writ_md.char_props.write = 1;
    char_writ_md.char_props.read = 1;
    char_read_md.char_props.write = 1;
    char_read_md.char_props.read = 1;
    char_data_md.char_props.write = 1;
    char_data_md.char_props.read = 1;

    ble_gatts_attr_md_t attr_writ_md, attr_read_md, attr_data_md;
    memset(&attr_writ_md, 0, sizeof(attr_writ_md));
    memset(&attr_read_md, 0, sizeof(attr_read_md));
    memset(&attr_data_md, 0, sizeof(attr_data_md));
    attr_writ_md.vloc        = BLE_GATTS_VLOC_STACK;
    attr_read_md.vloc        = BLE_GATTS_VLOC_STACK;
    attr_data_md.vloc        = BLE_GATTS_VLOC_STACK;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_writ_md.write_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_writ_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_read_md.write_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_read_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_data_md.write_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_data_md.read_perm);

    /* config characteristic value attr */
    ble_gatts_attr_t    attr_char_writ_value;
    ble_gatts_attr_t    attr_char_read_value;
    ble_gatts_attr_t    attr_char_data_value;
    memset(&attr_char_writ_value, 0, sizeof(attr_char_writ_value));    
    memset(&attr_char_read_value, 0, sizeof(attr_char_read_value));  
    memset(&attr_char_data_value, 0, sizeof(attr_char_data_value));  

    attr_char_writ_value.p_uuid      = &char_writ_uuid;
    attr_char_read_value.p_uuid      = &char_read_uuid;
    attr_char_data_value.p_uuid      = &char_data_uuid;

    attr_char_writ_value.p_attr_md   = &attr_writ_md;
    attr_char_read_value.p_attr_md   = &attr_read_md;
    attr_char_data_value.p_attr_md   = &attr_data_md;

    attr_char_writ_value.max_len     = 1;
    attr_char_writ_value.init_len    = 1;
    attr_char_read_value.max_len     = 1;
    attr_char_read_value.init_len    = 1;
    attr_char_data_value.max_len     = 1;
    attr_char_data_value.init_len    = 1;

    uint8_t value[10]            = {0,0,0,0,0,0,0,0,0,0};
                                    //0,0,0,0,0,0,0,0,0,0,
                                    //0,0,0,0,0,0,0,0,0,0,
                                    //0,0};
    attr_char_writ_value.p_value     = value;
    attr_char_read_value.p_value     = value;
    attr_char_data_value.p_value     = value;

    err_code = sd_ble_gatts_characteristic_add(p_tonepen_service->service_handle,
                                   &char_writ_md,
                                   &attr_char_writ_value,
                                   &p_tonepen_service->char_writ_handles);
                                   
    APP_ERROR_CHECK(err_code);

    
    err_code = sd_ble_gatts_characteristic_add(p_tonepen_service->service_handle,
                                   &char_read_md,
                                   &attr_char_read_value,
                                   &p_tonepen_service->char_read_handles);
                                  
    APP_ERROR_CHECK(err_code);
    return;
    err_code = sd_ble_gatts_characteristic_add(p_tonepen_service->service_handle,
                                   &char_data_md,
                                   &attr_char_data_value,
                                   &p_tonepen_service->char_data_handles);
    APP_ERROR_CHECK(err_code);

    return ;
}
void tonepen_service_init(ble_tonepen_t * p_tonepen_service, nrf_twi_mngr_t *twi_mngr)
{
    uint32_t err_code;
    
    
    
    // STEP 3: Declare 16 bit service and 128 bit base UUIDs and add them to BLE stack table     
 
    ble_uuid_t    service_uuid;
    ble_uuid128_t base_uuid = BLE_UUID_TONEPEN_BASE_UUID;
    service_uuid.uuid = BLE_UUID_TONEPEN_SERVICE;
    err_code = sd_ble_uuid_vs_add(&base_uuid, &service_uuid.type);
    APP_ERROR_CHECK(err_code);

    p_tonepen_service->conn_handle = BLE_CONN_HANDLE_INVALID;


    // STEP 4: Add our service
    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &service_uuid, &p_tonepen_service->service_handle);
    APP_ERROR_CHECK(err_code);

    SEGGER_RTT_WriteString(0, "Executing tonepen_service_init().\n"); // Print message to RTT to the application flow
    SEGGER_RTT_printf(0, "Service UUID: 0x%#04x\n", service_uuid.uuid); // Print service UUID should match definition BLE_UUID_OUR_SERVICE
    SEGGER_RTT_printf(0, "Service UUID type: 0x%#02x\n", service_uuid.type); // Print UUID type. Should match BLE_UUID_TYPE_VENDOR_BEGIN. Search for BLE_UUID_TYPES in ble_types.h for more info
    SEGGER_RTT_printf(0, "Service handle: 0x%#04x\n", p_tonepen_service->service_handle); // Print out the service handle. Should match service handle shown in MCP under Attribute values
 
    tonepen_add_tmd_characteristics(&base_uuid, p_tonepen_service);
   // APP_ERROR_CHECK(err_code);


//Print messages to Segger Real Time Terminal
    //UNCOMMENT THE FOUR LINES BELOW AFTER INITIALIZING THE SERVICE OR THE EXAMPLE WILL NOT COMPILE.
    
    /* Add Tonepen Light sensor Characteristic UUID *
    ble_uuid_t          char_uuid;
    char_uuid.uuid      = BLE_UUID_TONEPEN_CHAR_LIGHT_UUID;
    err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
    APP_ERROR_CHECK(err_code);  

     Config permissions *
    ble_gatts_char_md_t char_md;
    memset(&char_md, 0, sizeof(char_md));
    char_md.char_props.read = 1;
    char_md.char_props.write = 1;

    ble_gatts_attr_md_t cccd_md;
    memset(&cccd_md, 0, sizeof(cccd_md));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
    cccd_md.vloc                = BLE_GATTS_VLOC_STACK;    
    char_md.p_cccd_md           = &cccd_md;
    char_md.char_props.notify   = 1;

    Config attrib metadata 
    ble_gatts_attr_md_t attr_md;
    memset(&attr_md, 0, sizeof(attr_md));
    attr_md.vloc        = BLE_GATTS_VLOC_STACK;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

     config characteristic value attr 
    ble_gatts_attr_t    attr_char_value;
    memset(&attr_char_value, 0, sizeof(attr_char_value));    
    attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md;

    attr_char_value.max_len     = 12;
    attr_char_value.init_len    = 12;
    uint8_t value[12]            = {0,0,0,0,0,0,0,0,0,0,0,0};
    attr_char_value.p_value     = value;

    /add characteristic to service 
    err_code = sd_ble_gatts_characteristic_add(p_tonepen_service->service_handle,
                                   &char_md,
                                   &attr_char_value,
                                   &p_tonepen_service->char_handles);
    APP_ERROR_CHECK(err_code);



    * Create Tonepen Characteristic for writing a TMD3702 register 
    //ble_uuid_t          char_uuid;
    char_uuid.uuid      = BLE_UUID_TONEPEN_CHAR_TMD3702_WRITE_UUID;
    err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
    APP_ERROR_CHECK(err_code);

   * Config permissions 
    //ble_gatts_char_md_t char_md;
    memset(&char_md, 0, sizeof(char_md));
    char_md.char_props.read = 1;
    char_md.char_props.write = 1;

    / Config attrib metadata 
    //ble_gatts_attr_md_t attr_md;
    memset(&attr_md, 0, sizeof(attr_md));
    attr_md.vloc        = BLE_GATTS_VLOC_STACK;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

    / config characteristic value attr 
    //ble_gatts_attr_t    attr_char_value;
    memset(&attr_char_value, 0, sizeof(attr_char_value));    
    attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md;

    attr_char_value.max_len     = 2;
    attr_char_value.init_len    = 2;
    uint8_t value2[2]            = {0,0};
    attr_char_value.p_value     = value2;

    / add characteristic to service 
    err_code = sd_ble_gatts_characteristic_add(p_tonepen_service->service_handle,
                                   &char_md,
                                   &attr_char_value,
                                   &p_tonepen_service->tmd3702_write_handles);
    APP_ERROR_CHECK(err_code);

    
    / Create Tonepen Characteristic for reading a TMD3702 register 
    //ble_uuid_t          char_uuid;
    char_uuid.uuid      = BLE_UUID_TONEPEN_CHAR_TMD3702_READ_UUID;
    err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
    APP_ERROR_CHECK(err_code);

    / Config permissions 
    //ble_gatts_char_md_t char_md;
    memset(&char_md, 0, sizeof(char_md));
    char_md.char_props.read = 1;
    char_md.char_props.write = 1;

    / Config attrib metadata 
    //ble_gatts_attr_md_t attr_md;
    memset(&attr_md, 0, sizeof(attr_md));
    attr_md.vloc        = BLE_GATTS_VLOC_STACK;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

     config characteristic value attr 
    //ble_gatts_attr_t    attr_char_value;
    memset(&attr_char_value, 0, sizeof(attr_char_value));    
    attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md;

    attr_char_value.max_len     = 2;
    attr_char_value.init_len    = 2;
    uint8_t value3[2]            = {0,0};
    attr_char_value.p_value     = value3;

    / add characteristic to service 
    err_code = sd_ble_gatts_characteristic_add(p_tonepen_service->service_handle,
                                   &char_md,
                                   &attr_char_value,
                                   &p_tonepen_service->tmd3702_read_handles);
    APP_ERROR_CHECK(err_code);*/
}
void ble_tonepen_service_on_ble_evt(ble_evt_t const * p_ble_evt, void * p_context) {

}


void tonepen_update(ble_tonepen_t *p_tonepen_service, int32_t *value) {
    if (p_tonepen_service->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        uint16_t               len = 12;
        ble_gatts_hvx_params_t hvx_params;
        memset(&hvx_params, 0, sizeof(hvx_params));
/*
        hvx_params.handle = p_tonepen_service->char_handles.value_handle;
        hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
        hvx_params.offset = 0;
        hvx_params.p_len  = &len;
        hvx_params.p_data = (uint8_t*)value;

        sd_ble_gatts_hvx(p_tonepen_service->conn_handle, &hvx_params);*/
    }
}